#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a drawing using G'mic Engrave
#   approaching the look of a real Engraving on a material
#   like wood, stone, or any flat hard coloured material 

# (c) Dinasset 2020
#
#   History:
#
#   v1.0: 2020-12-01: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *

# define here some constant value, when required
wrkwh=3000
nr_of_colours=12    #32
# predefined colour names
BLOSSOM=(250,198,204,255)
CORAL=(255,182,173,255)
PAPAYA=(246,139,105,255)
POPPY=(251,175,95,255)
CURRY=(250,207,99,255)
SUNSHINE=(255,239,108,255)
CHARTREUSE=(206,215,92,255)
MINT=(210,239,219,255)
CLOVER=(161,206,94,255)
SAGE=(219,223,195,255)
BLUEBELL=(174,209,233,255)
POOL=(162,208,207,255)
PLUM=(204,193,219,255)
CEMENT=(227,223,214,255)
GRAVEL=(178,177,165,255)
PAPER_BAG=(197,171,137,255)
LUXE_BLUSH=(255,242,236,255)
ANTIQUE_GOLD=(179,142,80,255)
SHIMMER_GOLD=(255,226,147,255)
SHIMMER_SILVER=(200,201,208,255)
# added
RED=(221,50,50,255)
FUCHSIA=(236,0,140,255)
PERSIMMON=(228,62,34,255)
MOSS=(150,154,82,255)
LAKE=(26,134,168,255)
PEACOCK=(0,145,156,255)
ROYAL_BLUE=(0,101,169,255)
NIGHT=(0,43,84,255)
BEET=(152,83,150,255)
FIG=(128,84,98,255)
AUBERGINE=(57,18,66,255)
SLATE=(88,87,75,255)
CHOCOLATE=(87,44,41,255)
#BLACK=(0,0,0,255)



############################################################################
# Service routines

#---------------------------------------------------------------------------
# for messages: include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
    
#---------------------------------------------------------------------------
# Palette to Gradient (modified from the existing std Gimp filter availagbe)
# the modification done permits the creation of a hard-border gradient
#---------------------------------------------------------------------------
def make_gradient_hard(palette, num_segments, num_colors):
    
    gradient = pdb.gimp_gradient_new(palette)

    if (num_segments > 1):
        pdb.gimp_gradient_segment_range_split_uniform(gradient, 0, -1,
                                                      num_segments)

    for color_number in range(0,num_segments):
        if (color_number == num_colors-1):color_number_next = 0
        else: color_number_next = color_number + 1
        color_left = pdb.gimp_palette_entry_get_color(palette,
                                                      color_number)
        color_right = pdb.gimp_palette_entry_get_color(palette,
                                                       color_number_next)
        pdb.gimp_gradient_segment_set_left_color(gradient,
                                                 color_number, color_left,
                                                 100.0)
        pdb.gimp_gradient_segment_set_right_color(gradient,
                                                  color_number, color_left,
                                                                #mod, was right 
                                                  100.0)
    pdb.gimp_context_set_gradient(gradient)
    return gradient


def palette_to_gradient_repeating_hard(palette):
    num_colors = pdb.gimp_palette_get_info(palette)
    num_segments = num_colors
    return make_gradient_hard(palette, num_segments, num_colors)

def palette_to_gradient_hard(palette):
    num_colors = pdb.gimp_palette_get_info(palette)
    num_segments = num_colors - 1
    return make_gradient_hard(palette, num_segments, num_colors)
    
#---------------------------------------------------------------------------
# Add a Frame filled with a Gradient generated using the Image colours
#---------------------------------------------------------------------------
def GradientFrameFromImage (image,layer,nr_of_colours,
                            framepct, fixedratio):
    
    # House-keeping
    # -------------
    width = layer.width
    height = layer.height
    WH = width+height
    source_name = image.name
    #gimp.message(str(source_name))
    split = source_name.split('(')   # when name comes from image
    source_name = split[0]
    initial = source_name[0:1]
    if initial=="[":    # std name is [name]; name attributed by ofnuts' filter is name without [] 
        end = len(source_name)-2
        source_name = source_name[1:end]    # remove []
    
    # Posterize and Create Palette
    # ----------------------------
    # remove previously generated palettes
    Pname="Generated by "+source_name
    PnameALL="Generated by "
    (nP,lP)=pdb.gimp_palettes_get_list(PnameALL)
    if nP>0:
        x=0
        for x in range(0,nP): 
            pdb.gimp_palette_delete (lP[x])
        #gimp.message("deleted "+str(nP)+" palette(s) generated in previous run(s)")    
        
    PaletteImage = pdb.gimp_image_new (width, height, RGB)                    
    PaletteBG = pdb.gimp_layer_new_from_drawable (layer, PaletteImage)
    PaletteImage.add_layer (PaletteBG, 0)
    PaletteBG.mode = LAYER_MODE_NORMAL
    
    pdb.gimp_image_convert_indexed (PaletteImage, 0, 0,
                                    nr_of_colours, 0, 0, "ignored")
       
    # get the number of pixels in the colormap (which exists for indexed images)
    (count, colors) = pdb.gimp_image_get_colormap (PaletteImage)
    # the count is nr_of_colors * 3, example 8*3=24
    nr_areas = (count/3)
    generated_palette = pdb.gimp_palette_new (Pname)
    index = 0
    while index < count:
        color = (colors[index], colors[index+1], colors[index+2])
        pdb.gimp_palette_add_entry (generated_palette, str(index/3), color)
        index+=3
    pdb.gimp_image_convert_rgb (PaletteImage)
    pdb.gimp_layer_add_alpha (PaletteBG)        #  RGBA
    pdb.gimp_selection_none (PaletteImage)
    #gimp.message ("created palette: "+str(generated_palette))
    pdb.gimp_palettes_refresh()
    
    # Create Gradient from Palette
    # remove previously generated palettes
    Gname="Generated by "+source_name
    GnameALL="Generated by "
    (nG,lG)=pdb.gimp_gradients_get_list(GnameALL)
    if nG>0:
        x=0
        for x in range(0,nG): 
            pdb.gimp_gradient_delete (lG[x])
        #gimp.message("deleted "+str(nG)+" gradient(s) generated in previous run(s)")    
    generated_gradient = palette_to_gradient_repeating_hard (generated_palette)    
    #gimp.message ("created gradient: "+str(generated_gradient))
    pdb.gimp_gradients_refresh()
    # added 2.12
    pdb.gimp_image_delete (PaletteImage)    # nno need anymore
    # ----------  
    # Create the Frame around the Image
    if fixedratio==True:
        FWidth = int(WH/2*framepct/100.0)
        FHeight = int(WH/2*framepct/100.0)
    else:
        FWidth = int(width*framepct/100.0)
        FHeight = int(height*framepct/100.0)
    FrameWidth = width+FWidth
    FrameHeight = height+FHeight
    FrameLayer = pdb.gimp_layer_new (image, FrameWidth, FrameHeight, RGBA_IMAGE,
                        "Gradient Layer", 100.0, LAYER_MODE_NORMAL)
    image.add_layer (FrameLayer, 0)
    pdb.gimp_layer_translate (FrameLayer, -FWidth/2, -FHeight/2)
    pdb.gimp_image_resize_to_layers (image)
    pdb.gimp_selection_all (image)
    pdb.gimp_image_select_rectangle (image, CHANNEL_OP_SUBTRACT,
                                     FWidth/2, FHeight/2, width, height)
    pdb.gimp_context_set_gradient (generated_gradient)
    pdb.gimp_edit_blend (FrameLayer, BLEND_CUSTOM, LAYER_MODE_NORMAL, GRADIENT_SHAPEBURST_DIMPLED, 100.0,
                         #drawable,    blend-mode,   paint-mode,        gradient-type,               opacity, 
                         0.0,REPEAT_NONE, True, False, 1, 0.0, False,
                         #offset,repeat,reverse,supers,depth,thresh,dither, 
                         FrameWidth/2,FrameHeight/2,FrameWidth, FrameHeight) 
                         #x1,          y1,           x2,         y2
    nr_of_grad_segm = pdb.gimp_gradient_get_number_of_segments (generated_gradient)
                          
    (nr_color_samples, color_samples) = pdb.gimp_gradient_get_uniform_samples (generated_gradient,
                                                                               nr_of_grad_segm, False)
    grad_color_samples = len(color_samples)/4
    #info_msg ("nr of colour samples="+str(grad_color_samples))
    gcol=0
    while gcol<grad_color_samples:
        #info_msg ("gt_bevel run #"+str(gcol))
        pdb.gimp_image_select_color (image, CHANNEL_OP_REPLACE, image.layers[0],
                       (color_samples[gcol*4],color_samples[gcol*4+1],color_samples[gcol*4+2],255))
    #                              RED                 GREEN                   BLUE
        pdb.script_fu_gt_bevel (image, FrameLayer, 1,
                            framepct, framepct, 90.0, framepct, 0.0,
    #                       bwidth bheight,ldir,ldist,amb
                            True, False, False)
    #                       stretch,invert,keep
        FrameLayer = image.layers[0]
        gcol+=2 # every other colour
    pdb.gimp_selection_none (image)
    
    return(FrameLayer)

#---------------------------------------------------------------------------
# from Original drawable create the Output image
#---------------------------------------------------------------------------
def createOutimage(inImage, inLayer, widthheight):
     
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    source_name = inImage.name
    #gimp.message(str(source_name))
    split = source_name.split('(')   # when name comes from image
    source_name = split[0]
    initial = source_name[0:1]
    if initial=="[":    # std name is [name]; name attributed by ofnuts' filter is name without [] 
        end = len(source_name)-2
        source_name = source_name[1:end]    # remove []
   # define/calculate the scale factor for the (new) output image -------------------------
    realWidthHeight = owidth+oheight      # real size of the input image
    if widthheight == 0 :   # means the user wants to have output size same as input size
        widthheight = realWidthHeight
    optiWidthHeight = widthheight       # size to work on to get best results
    scale = (optiWidthHeight*1.0)/(realWidthHeight*1.0)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    
   # create a new image with the fixed work-size ---
   # --- copy the source layer
   # --- scale
    newImage = pdb.gimp_image_new (rwidth, rheight, otype)
    pdb.gimp_image_set_filename (newImage, source_name)
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "BG Layer from "+oname
    newBG.opacity = 100.0 
    newImage.add_layer(newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    pdb.gimp_layer_add_alpha (newBG)    # RGBA
   # keep a copy of the original background below the one which will be used ----------------
   # --- and set its visibility off (it should not enter in the process)
    Original = newBG.copy()
    newImage.add_layer(Original, 1)     # below
    pdb.gimp_item_set_visible (Original, False)
    Original.name = oname+" (colour=original)"
     
    newBG.opacity = 100.0
    
    return(newImage)

#---------------------------------------------------------------------------
# adjust values
#---------------------------------------------------------------------------
def adjustmeanv(inImage, inLayer):
    # check mean value of the layer
    (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.0, 1.0)
    mofified=False
    if meanl < 110:
        #info_msg("mean value of pattern = "+str(meanl))
        if meanl < 80: # avoid excessive loops
            pdb.gimp_drawable_curves_spline (inLayer, HISTOGRAM_VALUE,
                                             6, (0.0,0.0, 0.3,0.6, 0.95,1.0))
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
        while meanl < 110:
            pdb.gimp_drawable_levels (inLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.05, 0.00, 1.00, False)
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
        mofified=True
        #info_msg("modified mean luminance = "+str(meanl))
    if meanl > 144:
        #info_msg("mean value found = "+str(meanl))
        if meanl > 208: # avoid excessive loops
            pdb.gimp_drawable_curves_spline (inLayer, HISTOGRAM_VALUE,
                                             6, (0.0,0.0, 0.5,0.375, 1.0,0.75))
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
        if meanl > 176: # avoid excessive loops
            pdb.gimp_drawable_curves_spline (inLayer, HISTOGRAM_VALUE,
                                             6, (0.0,0.0, 0.5,0.45, 1.0,0.90))
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
        while meanl > 144:
            pdb.gimp_drawable_levels (inLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.95, 0.00, 1.00, False)
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
        mofified=True
        #info_msg("modified mean value = "+str(meanl))
    
    return(inLayer)
     
#---------------------------------------------------------------------------
# create main contour lines
#---------------------------------------------------------------------------
def maincontour(inImage, inLayer):
    #
    tmpImage = pdb.gimp_image_new (inImage.width, inImage.height, inImage.base_type)                    
    tmpBG = pdb.gimp_layer_new_from_drawable(inLayer, tmpImage)  # background layer
    tmpImage.add_layer (tmpBG, 0)
    tmpBG.mode = LAYER_MODE_NORMAL
     
    pdb.gimp_drawable_equalize (tmpBG, False)    
    
    tmpBLUR = tmpBG.copy()
    tmpImage.add_layer (tmpBLUR, 0)
    pdb.plug_in_gauss (tmpImage, tmpBLUR, tmpBLUR.width/50,     
                       tmpBLUR.height/50, 0)
    tmpBLUR.mode = LAYER_MODE_DIVIDE
    
    tmpCONTOURS = pdb.gimp_layer_new_from_visible (tmpImage, tmpImage,"CONTOURS")
    tmpImage.add_layer (tmpCONTOURS, 0)
     
    pdb.gimp_drawable_desaturate (tmpCONTOURS, DESATURATE_LUMA)     # add 28.11
    pdb.gimp_drawable_curves_spline (tmpCONTOURS, HISTOGRAM_VALUE,  # add 28.11
                                     8, (0.0,0.0, 0.25,0.10, 0.75,0.20, 1.0,1.0))
    
    #pdb.gimp_display_new (tmpImage)
    
    return(tmpCONTOURS)
     
#--------------------------------------------------------------------------- 
    

############################################################################
# Main
#---------------------------------------------------------------------------

def DrawStyleL (inImage, inLayer,
                 widthheight,
                 BGpredefcol, BGusercol, BGpattern,
                 repeatOnTop,
                 doFrame, doFlatten):
    
    pdb.gimp_context_push()
     
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    if widthheight == 0 :   # the user wants to have output size same as input size
        widthheight = inImage.width+inImage.height   # real size of the input image
     
    newImage = createOutimage (inImage, inLayer, wrkwh)
    Created = newImage.layers[0]
    Original = newImage.layers[1]
    # suffix
    suffix = " as a drawing engraved on "+BGpattern 
        
    Preprocessed = Created.copy()
    newImage.add_layer (Preprocessed, 0)
    PpW = Preprocessed.width
    PpH = Preprocessed.height
    PpWH = Preprocessed.width+Preprocessed.height
    pdb.plug_in_sel_gauss (newImage, Preprocessed, PpWH/125, 50)
    pdb.gimp_drawable_posterize (Preprocessed, nr_of_colours)   
    Preprocessed.name = Created.name+"-posterized"
    
    # check and adjust mean value
    Preprocessed = adjustmeanv(newImage, Preprocessed)
    # sel all values in the range 0.4-1.0
    pdb.gimp_drawable_curves_spline (Preprocessed, HISTOGRAM_VALUE,
                                     6, (0.0,0.4, 0.4,0.8, 1.0,1.0))
    
    #pdb.gimp_display_new (newImage)
    #return
    
    #[G'MIC] Engrave: fx_engrave 0.5,50,0,8,40,0,0,0,10,1,0,0,0,1,0,50,50  # std
    pdb.plug_in_gmic_qt(newImage, Preprocessed, 1,2,    
        "-v - -fx_engrave 0.9,100,0,8,40,0,0,0,10,1,0,0,0,1,0,50,50 ")
    Engraved1 = newImage.layers[0]
    pdb.gimp_layer_add_alpha (Engraved1)    # RGBA
    pdb.plug_in_erode (newImage, Engraved1, 1, 7, 1.0, 7, 0, 80)
    pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE, Engraved1, (255,255,255,255))
    pdb.gimp_edit_cut (Engraved1)
    pdb.gimp_selection_none (newImage)
    Engraved1.name = Preprocessed.name+"-engraved (positive)"
    Engraved1.mode = LAYER_MODE_BURN     
    Engraved1.opacity = 50.0             
    
    #grant main contours
    pdb.gimp_item_set_visible (Original, True)  # activate visibility temp
    tmpCONTOURS=maincontour(newImage, Original)
    MainContours = pdb.gimp_layer_new_from_drawable (tmpCONTOURS, newImage)
    newImage.add_layer (MainContours, 0)
    MainContours.name = "Main Contours"
     
    pdb.plug_in_colortoalpha (newImage, MainContours, (255,255,255,255))    # add 28.11
    pdb.gimp_item_set_visible (Original, False)
        
    #pdb.gimp_display_new (newImage)
    #return
    
    # create coloured BG 
    if BGpredefcol == 0: pdb.gimp_context_set_foreground (BGusercol); po=" (colour=user)"
    if BGpredefcol == 1: pdb.gimp_context_set_foreground (BLOSSOM); po=" (colour=blossom)"
    if BGpredefcol == 2: pdb.gimp_context_set_foreground (CORAL); po=" (colour=coral)"
    if BGpredefcol == 3: pdb.gimp_context_set_foreground (PAPAYA); po=" (colour=papaya)"
    if BGpredefcol == 4: pdb.gimp_context_set_foreground (POPPY); po=" (colour=poppy)"
    if BGpredefcol == 5: pdb.gimp_context_set_foreground (CURRY); po=" (colour=curry)"
    if BGpredefcol == 6: pdb.gimp_context_set_foreground (SUNSHINE); po=" (colour=sunshine)"
    if BGpredefcol == 7: pdb.gimp_context_set_foreground (CHARTREUSE); po=" (colour=chartreuse)"
    if BGpredefcol == 8: pdb.gimp_context_set_foreground (MINT); po=" (colour=mint)"
    if BGpredefcol == 9: pdb.gimp_context_set_foreground (CLOVER); po=" (colour=clover)"
    if BGpredefcol == 10: pdb.gimp_context_set_foreground (SAGE); po=" (colour=sage)"
    if BGpredefcol == 11: pdb.gimp_context_set_foreground (BLUEBELL); po=" (colour=bluebell)"
    if BGpredefcol == 12: pdb.gimp_context_set_foreground (POOL); po=" (colour=pool)"
    if BGpredefcol == 13: pdb.gimp_context_set_foreground (PLUM); po=" (colour=plum)"
    if BGpredefcol == 14: pdb.gimp_context_set_foreground (CEMENT); po=" (colour=cement)"
    if BGpredefcol == 15: pdb.gimp_context_set_foreground (GRAVEL); po=" (colour=gravel)"
    if BGpredefcol == 16: pdb.gimp_context_set_foreground (PAPER_BAG); po=" (colour=paper-bag)"
    if BGpredefcol == 17: pdb.gimp_context_set_foreground (LUXE_BLUSH); po=" (colour=luxe-blush)"
    if BGpredefcol == 18: pdb.gimp_context_set_foreground (ANTIQUE_GOLD); po=" (colour=antique-gold)"
    if BGpredefcol == 19: pdb.gimp_context_set_foreground (SHIMMER_GOLD); po=" (colour=shimmer-gold)"
    if BGpredefcol == 20: pdb.gimp_context_set_foreground (SHIMMER_SILVER); po=" (colour=shimmer-silver)"
     
    if BGpredefcol == 21: pdb.gimp_context_set_foreground (RED); po=" (colour=red)"
    if BGpredefcol == 22: pdb.gimp_context_set_foreground (FUCHSIA); po=" (colour=fuchsia)"
    if BGpredefcol == 23: pdb.gimp_context_set_foreground (PERSIMMON); po=" (colour=persimmon)"
    if BGpredefcol == 24: pdb.gimp_context_set_foreground (MOSS); po=" (colour=moss)"
    if BGpredefcol == 25: pdb.gimp_context_set_foreground (LAKE); po=" (colour=lake)"
    if BGpredefcol == 26: pdb.gimp_context_set_foreground (PEACOCK); po=" (colour=peacock)"
    if BGpredefcol == 27: pdb.gimp_context_set_foreground (ROYAL_BLUE); po=" (colour=royal-blue)"
    if BGpredefcol == 28: pdb.gimp_context_set_foreground (NIGHT); po=" (colour=night)"
    if BGpredefcol == 29: pdb.gimp_context_set_foreground (BEET); po=" (colour=beet)"
    if BGpredefcol == 30: pdb.gimp_context_set_foreground (FIG); po=" (colour=fig)"
    if BGpredefcol == 31: pdb.gimp_context_set_foreground (AUBERGINE); po=" (colour=aubergine)"
    if BGpredefcol == 32: pdb.gimp_context_set_foreground (SLATE); po=" (colour=slate)"
    if BGpredefcol == 33: pdb.gimp_context_set_foreground (CHOCOLATE); po=" (colour=chocolate)"
    
    suffix=suffix+po
    BG = pdb.gimp_layer_new (newImage, PpW, PpH, RGB_IMAGE,
                             "BG"+po, 80.0, LAYER_MODE_LINEAR_BURN)     
    newImage.add_layer (BG, 2)
    pdb.gimp_drawable_edit_fill (BG, FILL_FOREGROUND)
    Pattern = pdb.gimp_layer_new (newImage, PpW, PpH, RGB_IMAGE,
                             "BG pattern "+BGpattern,
                             35.0, LAYER_MODE_GRAIN_MERGE)
    newImage.add_layer (Pattern, 0)
    pdb.gimp_context_set_pattern (BGpattern)
    pdb.gimp_drawable_edit_fill (Pattern, FILL_PATTERN)
    # check/adjust mean value
    Pattern = adjustmeanv(newImage, Pattern)
    
    #pdb.gimp_display_new (newImage)
    #return
    
    Engraved1.opacity = 100.0
    Engrave1 = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                "Engrave (positive)")
    newImage.add_layer (Engrave1, 0)
    
    #pdb.gimp_display_new (newImage)
    #return

    Engrave1B=Engrave1.copy()
    newImage.add_layer (Engrave1B, 0)
    Engrave1B.name = "Engrave patterned (soft)"
    Engrave1B.mode=LAYER_MODE_SOFTLIGHT                     
    Engrave1B.opacity = 85.0
     
    pdb.plug_in_gauss (newImage, Engrave1B, 15.0, 15.0, 1)
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # final touch up  
    postfix = ""
    if repeatOnTop>0: # engraved on top
        postfix=", engraving on top"
        EngravedOver = Engraved1.copy()
        newImage.add_layer(EngravedOver, 0)     
        MainContoursOver = MainContours.copy()
        newImage.add_layer(MainContoursOver, 0)     
        # combine
        EngraveOver = pdb.gimp_image_merge_down (newImage,
                                                 MainContoursOver, CLIP_TO_BOTTOM_LAYER)
        EngraveOver.name = "Engraving on top"
        EngraveOver.mode = LAYER_MODE_GRAIN_MERGE    
        EngraveOver.opacity = 95.0                   
        if repeatOnTop>=2: # embossed/bumpmapped
            pdb.gimp_context_set_brush_force (1.0)
            if repeatOnTop==2:
                EB=1
                ppf=" (embossed)"
            if repeatOnTop==3:
                EB=0
                ppf=" (bumpmapped)"
            pdb.plug_in_emboss (newImage, EngraveOver, 90.0, 0.0, 90, EB)
            postfix=postfix+ppf
            EngraveOver.name=EngraveOver.name+ppf
        else:   # 1
            pdb.plug_in_dilate (newImage, EngraveOver, 0, 7, 1.0, 7, 0, 255)     
    else: # brighten
        Engrave1B.mode=LAYER_MODE_SCREEN
        pdb.gimp_drawable_curves_spline (Engrave1B, HISTOGRAM_VALUE,
                                     6, (0.0,0.1, 0.2,0.5, 1.0,0.8))

    #  reinforce the pattern  
    Pattern2 = Pattern.copy()
    newImage.add_layer (Pattern2, 0)
    Pattern2.mode=LAYER_MODE_OVERLAY          
    Pattern2.opacity = 100.0                  
    Pattern.name = "BG pattern "+BGpattern+" (colour=for GM)"
    
    # final step
    if doFlatten==True:
        outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_BOTTOM_LAYER)
        outcome.name = oname+suffix+postfix
    else:    
        outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
        newImage.add_layer (outcome, 0)
        #pdb.gimp_item_set_visible (outcome, False)
    
    # Frame?    
    if doFrame==True:
        Frame = GradientFrameFromImage (newImage,outcome,
                                        nr_of_colours,
                                        30, True)
        postfix = postfix+" (framed)"
        
    outcome.name = st+oname+suffix+postfix
    if widthheight<>wrkwh:   # 5.12
        uratio = widthheight*1.0/wrkwh*1.0
        uwidth = int(PpW*uratio); uheight = int(PpH*uratio)
        pdb.gimp_image_scale (newImage, uwidth, uheight)   # to user desired size
    pdb.gimp_display_new (newImage)
    pdb.gimp_context_pop()
     
### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/Drawing/"
copyrightYear="2020"
desc="Draw Effect Style L "

register(
    "DrawStyleL",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+"...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_ADJUSTMENT, "newsize", "Width+Height of New Image"+
                                   "\n (0=same as input)", 3000, (0, 7200, 100)),
        (PF_OPTION, "precol", "Predefined Base colour", 0, ["*user (choose below)",
                                                            "BLOSSOM=(250,198,204)",
                                                            "CORAL=(255,182,173)",
                                                            "PAPAYA=(246,139,105)",
                                                            "POPPY=(251,175,95)",
                                                            "CURRY=(250,207,99)",
                                                            "SUNSHINE=(255,239,108)",
                                                            "CHARTREUSE=(206,215,92)",
                                                            "MINT=(210,239,219)",
                                                            "CLOVER=(161,206,94)",
                                                            "SAGE=(219,223,195)",
                                                            "BLUEBELL=(174,209,233)",
                                                            "POOL=(162,208,207)",
                                                            "PLUM=(204,193,219)",
                                                            "CEMENT=(227,223,214)",
                                                            "GRAVEL=(178,177,165)",
                                                            "PAPER_BAG=(197,171,137)",
                                                            "LUXE_BLUSH=(255,242,236)",
                                                            "ANTIQUE_GOLD=(179,142,80)",
                                                            "SHIMMER_GOLD=(255,226,147)",
                                                            "SHIMMER_SILVER=(200,201,208)",
                                                            # added
                                                            "RED=(221,50,50)",
                                                            "FUCHSIA=(236,0,140)",
                                                            "PERSIMMON=(228,62,34)",
                                                            "MOSS=(150,154,82)",
                                                            "LAKE=(26,134,168)",
                                                            "PEACOCK=(0,145,156)",
                                                            "ROYAL_BLUE=(0,101,169)",
                                                            "NIGHT=(0,43,84)",
                                                            "BEET=(152,83,150)",
                                                            "FIG=(128,84,98)",
                                                            "AUBERGINE=(57,18,66)",
                                                            "SLATE=(88,87,75)",
                                                            "CHOCOLATE=(87,44,41)",
                                                            ]),
        (PF_COLOR, "freecol", "*user Base colour"+
                            "\n (use medium colours)", (150, 150, 150, 255)),
        (PF_PATTERN, "bgpattern", "Base pattern", "Pine"),
        (PF_OPTION, "repeatontop", "Reinforce Engraving on Top", 0, ("*no",     
                                                                     "*yes, simple",
                                                                     "*yes, embossed",
                                                                     "*yes, bumpmapped",
                                                                    )),
        (PF_TOGGLE, "frame", "Add Gradient Frame?"+
                             "\n (reqd GTbevel)", False),
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    DrawStyleL,
    menu=menu
)

main()
